<?php

/*
 * This file is part of the TaskManagementBundle for Kimai 2.
 * All rights reserved by Kevin Papst (www.kevinpapst.de).
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace KimaiPlugin\TaskManagementBundle\API;

use App\API\BaseApiController;
use App\API\NotFoundException;
use FOS\RestBundle\Controller\Annotations\RouteResource;
use FOS\RestBundle\View\View;
use FOS\RestBundle\View\ViewHandlerInterface;
use KimaiPlugin\TaskManagementBundle\Entity\Task;
use KimaiPlugin\TaskManagementBundle\Repository\TaskService;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Swagger\Annotations as SWG;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * @RouteResource("Task")
 */
class TaskController extends BaseApiController
{
    /**
     * @var ViewHandlerInterface
     */
    private $viewHandler;
    /**
     * @var TaskService
     */
    private $tasks;

    public function __construct(ViewHandlerInterface $viewHandler, TaskService $tasks)
    {
        $this->viewHandler = $viewHandler;
        $this->tasks = $tasks;
    }

    /**
     * Start a task record
     *
     * @SWG\Response(
     *      response=200,
     *      description="Start working on a task",
     *      @SWG\Schema(ref="#/definitions/TaskEntity")
     * )
     * @SWG\Parameter(
     *      name="id",
     *      in="path",
     *      type="integer",
     *      description="Task ID to start",
     *      required=true,
     * )
     *
     * @Security("is_granted('task_start')")
     */
    public function startAction(string $id): Response
    {
        $task = $this->tasks->findTask($id);

        if (null === $task) {
            throw new NotFoundException();
        }

        if (null === $task->getUser() || $this->getUser() !== $task->getUser()) {
            throw new AccessDeniedHttpException('Task cannot be started, not assigned to the user');
        }

        if (null !== $task->getActiveRecord()) {
            throw new BadRequestHttpException('Task cannot be started, already running');
        }

        $this->tasks->startTask($task);

        $view = new View(null, Response::HTTP_NO_CONTENT);

        return $this->viewHandler->handle($view);
    }

    /**
     * Stops a task record
     *
     * @SWG\Response(
     *      response=200,
     *      description="Stop working on a task",
     *      @SWG\Schema(ref="#/definitions/TaskEntity")
     * )
     * @SWG\Parameter(
     *      name="id",
     *      in="path",
     *      type="integer",
     *      description="Task ID to stop",
     *      required=true,
     * )
     *
     * @Security("is_granted('task_start')")
     */
    public function stopAction(string $id): Response
    {
        $task = $this->tasks->findTask($id);

        if (null === $task) {
            throw new NotFoundException();
        }

        if (null === $task->getUser() || $this->getUser() !== $task->getUser()) {
            throw new AccessDeniedHttpException('Task cannot be stopped, not assigned to the user');
        }

        if (null === $task->getActiveRecord()) {
            throw new BadRequestHttpException('Task cannot be stopped, not running yet');
        }

        $this->tasks->stopTask($task);

        $view = new View(null, Response::HTTP_NO_CONTENT);

        return $this->viewHandler->handle($view);
    }

    /**
     * Delete an existing task record
     *
     * @SWG\Delete(
     *      @SWG\Response(
     *          response=204,
     *          description="Delete one task record"
     *      ),
     * )
     * @SWG\Parameter(
     *      name="id",
     *      in="path",
     *      type="integer",
     *      description="Task record ID to delete",
     *      required=true,
     * )
     *
     * @Security("is_granted('task_delete')")
     */
    public function deleteAction(string $id): Response
    {
        $task = $this->tasks->findTask($id);

        if (null === $task) {
            throw new NotFoundException();
        }

        $this->tasks->deleteTask($task);

        $view = new View(null, Response::HTTP_NO_CONTENT);

        return $this->viewHandler->handle($view);
    }

    /**
     * Assign a task to the current user
     *
     * @SWG\Patch(
     *      description="Assign a currently unassigned task to the current user",
     *      @SWG\Response(
     *          response=200,
     *          description="Returns the updated task",
     *          @SWG\Schema(ref="#/definitions/TaskEntity")
     *      )
     * )
     * @SWG\Parameter(
     *      name="id",
     *      in="path",
     *      type="integer",
     *      description="Task ID to update",
     *      required=true,
     * )
     *
     * @Security("is_granted('task_assign')")
     */
    public function assignAction(string $id): Response
    {
        $task = $this->tasks->findTask($id);

        if (null === $task) {
            throw new NotFoundException();
        }

        if (null !== $task->getUser()) {
            throw new AccessDeniedHttpException('User cannot assign task, already assigned');
        }

        $this->tasks->assignTask($task, $this->getUser());

        $view = new View($task, Response::HTTP_OK);
        $view->getContext()->setGroups(['Default', 'Entity', 'Task']);

        return $this->viewHandler->handle($view);
    }

    /**
     * Unassign a task from the current user
     *
     * @SWG\Patch(
     *      description="Unassign a currently assigned task from the current user",
     *      @SWG\Response(
     *          response=200,
     *          description="Returns the updated task",
     *          @SWG\Schema(ref="#/definitions/TaskEntity")
     *      )
     * )
     * @SWG\Parameter(
     *      name="id",
     *      in="path",
     *      type="integer",
     *      description="Task ID to update",
     *      required=true,
     * )
     *
     * @Security("is_granted('task_assign')")
     */
    public function unassignAction(string $id): Response
    {
        $task = $this->tasks->findTask($id);

        if (null === $task) {
            throw new NotFoundException();
        }

        if ($this->getUser() !== $task->getUser()) {
            throw new AccessDeniedHttpException('User cannot unassign task, not the assignee');
        }

        $this->tasks->unassignTask($task);

        $view = new View($task, Response::HTTP_OK);
        $view->getContext()->setGroups(['Default', 'Entity', 'Task']);

        return $this->viewHandler->handle($view);
    }

    /**
     * Close a task for the current user
     *
     * @SWG\Patch(
     *      description="Closes an assigned task for the current user",
     *      @SWG\Response(
     *          response=200,
     *          description="Returns the closed task",
     *          @SWG\Schema(ref="#/definitions/TaskEntity")
     *      )
     * )
     * @SWG\Parameter(
     *      name="id",
     *      in="path",
     *      type="integer",
     *      description="Task ID to close",
     *      required=true,
     * )
     *
     * @Security("is_granted('task_close')")
     */
    public function closeAction(string $id): Response
    {
        $task = $this->tasks->findTask($id);

        if (null === $task) {
            throw new NotFoundException();
        }

        if ($this->getUser() !== $task->getUser()) {
            throw new AccessDeniedHttpException('User cannot close task');
        }

        if ($task->getStatus() === Task::STATUS_DONE) {
            throw new BadRequestHttpException('Task already closed');
        }

        $this->tasks->closeTask($task);

        $view = new View($task, Response::HTTP_OK);
        $view->getContext()->setGroups(['Default', 'Entity', 'Task']);

        return $this->viewHandler->handle($view);
    }

    /**
     * Reopens a task for the current user
     *
     * @SWG\Patch(
     *      description="Reopens an assigned task for the current user",
     *      @SWG\Response(
     *          response=200,
     *          description="Returns the reopened task",
     *          @SWG\Schema(ref="#/definitions/TaskEntity")
     *      )
     * )
     * @SWG\Parameter(
     *      name="id",
     *      in="path",
     *      type="integer",
     *      description="Task ID to reopen",
     *      required=true,
     * )
     *
     * @Security("is_granted('task_close')")
     */
    public function reopenAction(string $id): Response
    {
        $task = $this->tasks->findTask($id);

        if (null === $task) {
            throw new NotFoundException();
        }

        if ($this->getUser() !== $task->getUser()) {
            throw new AccessDeniedHttpException('User cannot reopen task');
        }

        if ($task->getStatus() !== Task::STATUS_DONE) {
            throw new BadRequestHttpException('Task cannot be reopened, not closed yet');
        }

        $this->tasks->reopenTask($task);

        $view = new View($task, Response::HTTP_OK);
        $view->getContext()->setGroups(['Default', 'Entity', 'Task']);

        return $this->viewHandler->handle($view);
    }
}
