<?php

/*
 * This file is part of the TaskManagementBundle for Kimai 2.
 * All rights reserved by Kevin Papst (www.kevinpapst.de).
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace KimaiPlugin\TaskManagementBundle\Controller;

use App\Controller\AbstractController;
use Doctrine\ORM\ORMException;
use KimaiPlugin\TaskManagementBundle\Entity\Task;
use KimaiPlugin\TaskManagementBundle\Form\TaskForm;
use KimaiPlugin\TaskManagementBundle\Form\TaskToolbarForm;
use KimaiPlugin\TaskManagementBundle\Repository\Query\TaskQuery;
use KimaiPlugin\TaskManagementBundle\Repository\TaskRepository;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route(path="/tasks")
 */
final class TaskController extends AbstractController
{
    /**
     * @var TaskRepository
     */
    private $repository;

    public function __construct(TaskRepository $repository)
    {
        $this->repository = $repository;
    }

    /**
     * @Route(path="/", defaults={"page": 1}, name="tasks", methods={"GET"})
     * @Route(path="/page/{page}", requirements={"page": "[1-9]\d*"}, name="tasks_paginated", methods={"GET"})
     * @Security("is_granted('task_view')")
     */
    public function index($page, Request $request): Response
    {
        $viewOtherUser = $this->isGranted('view_other_timesheet');

        $query = new TaskQuery();
        $query->setCurrentUser($this->getUser());
        if (!$viewOtherUser) {
            $query->setUser($this->getUser());
        }
        $query->setStatus([Task::STATUS_PENDING, Task::STATUS_IN_PROGRESS]);
        $query->setPage($page);

        $form = $this->getToolbarForm($query, $viewOtherUser);
        $form->setData($query);
        $form->submit($request->query->all(), false);

        if (!$form->isValid()) {
            $query->resetByFormError($form->getErrors());
        }

        if (null !== $query->getBegin()) {
            $query->getBegin()->setTime(0, 0, 0);
        }
        if (null !== $query->getEnd()) {
            $query->getEnd()->setTime(23, 59, 59);
        }

        $pager = $this->repository->getPagerfantaForQuery($query);

        return $this->render('@TaskManagement/index.html.twig', [
            'entries' => $pager,
            'page' => $query->getPage(),
            'query' => $query,
            'toolbarForm' => $form->createView(),
            'seeUserColumn' => $viewOtherUser,
        ]);
    }

    /**
     * @Route(path="/{id}/edit", name="tasks_edit", methods={"GET", "POST"})
     * @Security("is_granted('task_edit', task)")
     */
    public function editAction(Task $task, Request $request): Response
    {
        $assign = $this->isGranted('task_edit_other');

        $form = $this->createForm(TaskForm::class, $task, [
            'action' => $this->generateUrl('tasks_edit', ['id' => $task->getId()]),
            'include_user' => $assign,
            'include_team' => $assign,
        ]);

        return $this->renderEditForm($task, $request, $form);
    }

    /**
     * @Route(path="/create", name="tasks_create", methods={"GET", "POST"})
     * @Security("is_granted('task_edit_other')")
     */
    public function createAction(Request $request): Response
    {
        $task = new Task();
        $assign = $this->isGranted('task_edit_other');

        $form = $this->createForm(TaskForm::class, $task, [
            'action' => $this->generateUrl('tasks_create'),
            'include_user' => $assign,
            'include_team' => $assign,
            'attr' => [
                'data-form-event' => 'kimai.taskUpdate kimai.taskCreate',
            ],
        ]);

        return $this->renderEditForm($task, $request, $form);
    }

    /**
     * @Route(path="/create_my", name="tasks_create_my", methods={"GET", "POST"})
     * @Security("is_granted('task_edit_own')")
     */
    public function createMyTaskAction(Request $request): Response
    {
        $task = new Task();
        $task->setUser($this->getUser());
        $assign = $this->isGranted('task_edit_other');

        $form = $this->createForm(TaskForm::class, $task, [
            'action' => $this->generateUrl('tasks_create_my'),
            'include_user' => $assign,
            'include_team' => $assign,
            'attr' => [
                'data-form-event' => 'kimai.taskUpdate kimai.myTaskCreate',
            ],
        ]);

        return $this->renderEditForm($task, $request, $form);
    }

    /**
     * @Route(path="/my", defaults={"page": 1}, name="tasks_my", methods={"GET", "POST"})
     * @Route(path="/my/{page}", requirements={"page": "[1-9]\d*"}, name="tasks_my_paginated", methods={"GET"})
     * @Security("is_granted('task_start')")
     */
    public function myTaskAction($page, Request $request): Response
    {
        return $this->render('@TaskManagement/user-tasks.html.twig', [
            'user' => $this->getUser(),
            'page' => $page,
        ]);
    }

    /**
     * @Route(path="/pending", defaults={"page": 1}, name="tasks_pending", methods={"GET", "POST"})
     * @Route(path="/pending/{page}", requirements={"page": "[1-9]\d*"}, name="tasks_pending_paginated", methods={"GET"})
     * @Security("is_granted('task_assign')")
     */
    public function pendingTaskAction($page, Request $request): Response
    {
        return $this->render('@TaskManagement/pending-tasks.html.twig', [
            'user' => $this->getUser(),
            'page' => $page,
        ]);
    }

    private function getToolbarForm(TaskQuery $query, bool $viewOtherUser): FormInterface
    {
        return $this->createForm(TaskToolbarForm::class, $query, [
            'action' => $this->generateUrl('tasks', [
                'page' => $query->getPage(),
            ]),
            'method' => 'GET',
            'include_user' => $viewOtherUser,
            'include_team' => $viewOtherUser,
        ]);
    }

    private function renderEditForm(Task $task, Request $request, FormInterface $editForm)
    {
        $editForm->handleRequest($request);

        if ($editForm->isSubmitted() && $editForm->isValid()) {
            try {
                $this->repository->saveTask($task);
                $this->flashSuccess('action.update.success');

                return $this->redirectToRoute('tasks');
            } catch (ORMException $ex) {
                $this->flashError('action.update.error', ['%reason%' => $ex->getMessage()]);
            }
        }

        return $this->render(
            '@TaskManagement/edit.html.twig',
            [
                'task' => $task,
                'form' => $editForm->createView(),
            ]
        );
    }
}
