<?php

/*
 * This file is part of the TaskManagementBundle for Kimai 2.
 * All rights reserved by Kevin Papst (www.kevinpapst.de).
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace KimaiPlugin\TaskManagementBundle\Entity;

use App\Entity\Activity;
use App\Entity\Project;
use App\Entity\Tag;
use App\Entity\Team;
use App\Entity\Timesheet;
use App\Entity\User;
use DateTime;
use DateTimeZone;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Table(name="kimai2_tasks",
 *     indexes={
 *          @ORM\Index(columns={"user_id", "status", "start_time", "end_time"}),
 *          @ORM\Index(columns={"team_id", "status", "start_time", "end_time"})
 *     }
 * )
 * @ORM\Entity(repositoryClass="KimaiPlugin\TaskManagementBundle\Repository\TaskRepository")
 */
class Task
{
    public const STATUS_PENDING = 'pending';
    public const STATUS_IN_PROGRESS = 'progress';
    public const STATUS_DONE = 'closed';

    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $id;
    /**
     * @var string
     *
     * @ORM\Column(name="title", type="string", length=100, nullable=false)
     * @Assert\NotBlank()
     * @Assert\Length(min=2, max=100)
     */
    private $title;
    /**
     * @var string
     *
     * @ORM\Column(name="status", type="string", length=20, nullable=false)
     * @Assert\NotBlank()
     * @Assert\Length(max=20)
     */
    private $status = self::STATUS_PENDING;
    /**
     * @var string
     *
     * @ORM\Column(name="description", type="text", nullable=true)
     */
    private $description;
    /**
     * @var Project
     *
     * @ORM\ManyToOne(targetEntity="App\Entity\Project")
     * @ORM\JoinColumn(onDelete="CASCADE", nullable=false)
     * @Assert\NotNull()
     */
    private $project;
    /**
     * @var Activity
     *
     * @ORM\ManyToOne(targetEntity="App\Entity\Activity")
     * @ORM\JoinColumn(onDelete="CASCADE", nullable=false)
     * @Assert\NotNull()
     */
    private $activity;
    /**
     * Assigned user that is working on that task.
     *
     * @var User
     *
     * @ORM\ManyToOne(targetEntity="App\Entity\User")
     * @ORM\JoinColumn(onDelete="CASCADE", nullable=true)
     */
    private $user;
    /**
     * If set, only this team can see the task.
     *
     * @var Team
     *
     * @ORM\ManyToOne(targetEntity="App\Entity\Team")
     * @ORM\JoinColumn(onDelete="CASCADE", nullable=true)
     */
    private $team;
    /**
     * @var DateTime
     *
     * @ORM\Column(name="start_time", type="datetime", nullable=true)
     */
    private $begin;
    /**
     * @var DateTime
     *
     * @ORM\Column(name="end_time", type="datetime", nullable=true)
     */
    private $end;
    /**
     * @var string
     *
     * @ORM\Column(name="timezone", type="string", length=64, nullable=true)
     */
    private $timezone;
    /**
     * @var bool
     */
    private $localized = false;
    /**
     * @var Tag[]|ArrayCollection
     *
     * @ORM\ManyToMany(targetEntity="App\Entity\Tag", cascade={"persist"})
     * @ORM\JoinTable(
     *  name="kimai2_task_tags",
     *  joinColumns={
     *      @ORM\JoinColumn(name="task_id", referencedColumnName="id", onDelete="CASCADE")
     *  },
     *  inverseJoinColumns={
     *      @ORM\JoinColumn(name="tag_id", referencedColumnName="id", onDelete="CASCADE")
     *  }
     * )
     */
    private $tags;
    /**
     * @var Timesheet[]|Collection
     *
     * @ORM\ManyToMany(targetEntity="App\Entity\Timesheet", cascade={"persist"})
     * @ORM\JoinTable(
     *  name="kimai2_task_timesheets",
     *  joinColumns={
     *      @ORM\JoinColumn(name="task_id", referencedColumnName="id", onDelete="CASCADE")
     *  },
     *  inverseJoinColumns={
     *      @ORM\JoinColumn(name="timesheet_id", referencedColumnName="id", onDelete="CASCADE", unique=true)
     *  }
     * )
     */
    private $timesheets;

    public function __construct()
    {
        $this->tags = new ArrayCollection();
        $this->timesheets = new ArrayCollection();
    }

    /**
     * Get entry id, returns null for new entities which were not persisted.
     *
     * @return int|null
     */
    public function getId(): ?int
    {
        return $this->id;
    }

    /**
     * Make sure begin and end date have the correct timezone.
     * This will be called once for each item after being loaded from the database.
     */
    protected function localizeDates()
    {
        if ($this->localized) {
            return;
        }

        if (null !== $this->begin) {
            $this->begin->setTimeZone(new DateTimeZone($this->timezone));
        }

        if (null !== $this->end) {
            $this->end->setTimeZone(new DateTimeZone($this->timezone));
        }

        $this->localized = true;
    }

    public function getBegin(): ?DateTime
    {
        $this->localizeDates();

        return $this->begin;
    }

    public function setBegin(?DateTime $begin): Task
    {
        $this->begin = $begin;

        if (null !== $begin) {
            $this->timezone = $begin->getTimezone()->getName();
        }

        return $this;
    }

    public function getEnd(): ?DateTime
    {
        $this->localizeDates();

        return $this->end;
    }

    public function setEnd(?DateTime $end): Task
    {
        $this->end = $end;

        if (null !== $end) {
            $this->timezone = $end->getTimezone()->getName();
        }

        return $this;
    }

    public function setUser(?User $user): Task
    {
        $this->user = $user;

        return $this;
    }

    public function getUser(): ?User
    {
        return $this->user;
    }

    public function setActivity(Activity $activity): Task
    {
        $this->activity = $activity;

        return $this;
    }

    public function getActivity(): ?Activity
    {
        return $this->activity;
    }

    public function setProject(Project $project): Task
    {
        $this->project = $project;

        return $this;
    }

    public function getProject(): ?Project
    {
        return $this->project;
    }

    public function setDescription(?string $description): Task
    {
        $this->description = $description;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function getTimezone(): string
    {
        return $this->timezone;
    }

    public function getTitle(): ?string
    {
        return $this->title;
    }

    public function setTitle(string $title): Task
    {
        $this->title = $title;

        return $this;
    }

    public function getTeam(): ?Team
    {
        return $this->team;
    }

    public function setTeam(?Team $team): Task
    {
        $this->team = $team;

        return $this;
    }

    public function getStatus(): string
    {
        return $this->status;
    }

    public function setStatus(string $status): Task
    {
        if (!in_array($status, [self::STATUS_PENDING, self::STATUS_IN_PROGRESS, self::STATUS_DONE])) {
            throw new \InvalidArgumentException(sprintf('Unknown task status: %s', $status));
        }
        $this->status = $status;

        return $this;
    }

    public function getActiveRecord(): ?Timesheet
    {
        /** @var Timesheet $timesheet */
        foreach ($this->timesheets->getIterator() as $timesheet) {
            if (null === $timesheet->getEnd()) {
                return $timesheet;
            }
        }

        return null;
    }

    public function addTimesheet(Timesheet $timesheet): Task
    {
        $this->timesheets->add($timesheet);

        return $this;
    }

    public function getTimesheets(): Collection
    {
        return $this->timesheets;
    }

    public function getDuration(): ?int
    {
        if ($this->timesheets->count() === 0) {
            return null;
        }

        $fullDuration = 0;

        /** @var Timesheet $timesheet */
        foreach ($this->timesheets->getIterator() as $timesheet) {
            $duration = $timesheet->getDuration();

            if (null === $timesheet->getEnd()) {
                $duration = time() - $timesheet->getBegin()->getTimestamp();
            }

            $fullDuration += $duration;
        }

        return $fullDuration;
    }

    public function isOverdue(): bool
    {
        if (null === $this->end) {
            return false;
        }

        return time() > $this->end->getTimestamp();
    }

    public function isClosed(): bool
    {
        return $this->status === self::STATUS_DONE;
    }
}
