<?php

/*
 * This file is part of the TaskManagementBundle for Kimai 2.
 * All rights reserved by Kevin Papst (www.kevinpapst.de).
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace KimaiPlugin\TaskManagementBundle\Form;

use App\Entity\Customer;
use App\Entity\Project;
use App\Form\FormTrait;
use App\Form\Type\DateTimePickerType;
use App\Form\Type\TeamType;
use App\Form\Type\UserType;
use App\Repository\CustomerRepository;
use App\Repository\ProjectRepository;
use App\Timesheet\UserDateTimeFactory;
use KimaiPlugin\TaskManagementBundle\Entity\Task;
use KimaiPlugin\TaskManagementBundle\Form\Type\StatusType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

class TaskForm extends AbstractType
{
    use FormTrait;

    /**
     * @var CustomerRepository
     */
    private $customers;
    /**
     * @var ProjectRepository
     */
    private $projects;
    /**
     * @var UserDateTimeFactory
     */
    protected $dateTime;

    /**
     * @param CustomerRepository $customer
     * @param ProjectRepository $project
     * @param UserDateTimeFactory $dateTime
     */
    public function __construct(CustomerRepository $customer, ProjectRepository $project, UserDateTimeFactory $dateTime)
    {
        $this->customers = $customer;
        $this->projects = $project;
        $this->dateTime = $dateTime;
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $activity = null;
        $project = null;
        $customer = null;
        $begin = null;
        $isNew = true;
        $user = null;
        $timezone = $this->dateTime->getTimezone()->getName();
        $entry = null;

        if (isset($options['data'])) {
            /** @var Task $entry */
            $entry = $options['data'];

            $activity = $entry->getActivity();
            $project = $entry->getProject();
            $customer = null === $project ? null : $project->getCustomer();

            if (null !== $entry->getId()) {
                $isNew = false;
            }

            if (null === $project && null !== $activity) {
                $project = $activity->getProject();
            }

            if (null !== ($begin = $entry->getBegin())) {
                $timezone = $begin->getTimezone()->getName();
            }
        }

        $dateTimeOptions = [
            'model_timezone' => $timezone,
            'view_timezone' => $timezone,
        ];

        $builder->add('title', TextType::class, [
            'label' => 'label.title',
            'required' => true,
            'help' => 'task.help.title',
        ]);

        /*
        $builder->add('begin', DateTimePickerType::class, array_merge($dateTimeOptions, [
            'label' => 'label.begin',
            'required' => false,
        ]));
        */

        $builder->add('end', DateTimePickerType::class, array_merge($dateTimeOptions, [
            'label' => 'label.end',
            'required' => false,
            'help' => 'task.help.end',
        ]));

        $this->addCustomer($builder, $customer);
        $this->addProject($builder, $isNew, $project, $customer);
        $this->addActivity($builder, $activity, $project);

        if (!$isNew && $entry->getStatus() === Task::STATUS_DONE) {
            $builder->add('status', StatusType::class, [
                'required' => true,
            ]);
        }

        $builder->add('description', TextareaType::class, [
            'label' => 'label.description',
            'required' => false,
        ]);

        if ($options['include_user'] && ($isNew || null === $entry->getActiveRecord())) {
            $builder->add('user', UserType::class, [
                'required' => false,
                'help' => 'task.help.user',
            ]);
        }

        if ($options['include_team'] && ($isNew || null === $entry->getActiveRecord())) {
            $builder->add('team', TeamType::class, [
                'placeholder' => '',
                'required' => false,
                'help' => 'task.help.team',
            ]);
        }
    }

    /**
     * {@inheritdoc}
     */
    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults([
            'data_class' => Task::class,
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            'csrf_token_id' => 'task_edit',
            'include_user' => false,
            'include_team' => false,
            'method' => 'POST',
            'attr' => [
                'data-form-event' => 'kimai.taskUpdate',
                'data-msg-success' => 'action.update.success',
                'data-msg-error' => 'action.update.error',
            ],
        ]);
    }
}
