<?php

/*
 * This file is part of the TaskManagementBundle for Kimai 2.
 * All rights reserved by Kevin Papst (www.kevinpapst.de).
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace KimaiPlugin\TaskManagementBundle\Repository\Query;

use App\Entity\Activity;
use App\Entity\Customer;
use App\Entity\Project;
use App\Entity\User;
use App\Form\Model\DateRange;
use App\Repository\Query\BaseQuery;

final class TaskQuery extends BaseQuery
{
    public const TASKS_ORDER_ALLOWED = [
        'title', 'customer', 'project', 'activity', 'description', 'status', 'user', 'begin', 'end', 'team'
    ];

    /**
     * @var Customer|null
     */
    private $customer;
    /**
     * @var Project|null
     */
    private $project;
    /**
     * @var Activity|null
     */
    private $activity;
    /**
     * @var User[]
     */
    private $users = [];
    /**
     * @var User
     */
    private $user = null;
    /**
     * @var DateRange
     */
    private $dateRange;
    /**
     * @var string[]
     */
    private $status = [];
    /**
     * @var bool
     */
    private $unassigned = false;

    public function __construct()
    {
        $this->setDefaults([
            'order' => self::ORDER_ASC,
            'orderBy' => 'end',
            'dateRange' => new DateRange(),
            'status' => [],
        ]);
    }

    public function addUser(User $user): TaskQuery
    {
        $this->users[$user->getId()] = $user;

        return $this;
    }

    public function removeUser(User $user): TaskQuery
    {
        if (isset($this->users[$user->getId()])) {
            unset($this->users[$user->getId()]);
        }

        return $this;
    }

    /**
     * @return User[]
     */
    public function getUsers(): array
    {
        return array_values($this->users);
    }

    public function getUser(): ?User
    {
        return $this->user;
    }

    public function setUser(User $user): TaskQuery
    {
        $this->user = $user;

        return $this;
    }

    public function getActivity(): ?Activity
    {
        return $this->activity;
    }

    public function setActivity(?Activity $activity): TaskQuery
    {
        $this->activity = $activity;

        return $this;
    }

    public function getCustomer(): ?Customer
    {
        return $this->customer;
    }

    public function setCustomer(?Customer $customer): TaskQuery
    {
        $this->customer = $customer;

        return $this;
    }

    public function getProject(): ?Project
    {
        return $this->project;
    }

    public function setProject(?Project $project): TaskQuery
    {
        $this->project = $project;

        return $this;
    }

    public function getBegin(): ?\DateTime
    {
        return $this->dateRange->getBegin();
    }

    public function setBegin(\DateTime $begin): TaskQuery
    {
        $this->dateRange->setBegin($begin);

        return $this;
    }

    public function getEnd(): ?\DateTime
    {
        return $this->dateRange->getEnd();
    }

    public function setEnd(\DateTime $end): TaskQuery
    {
        $this->dateRange->setEnd($end);

        return $this;
    }

    public function getDateRange(): DateRange
    {
        return $this->dateRange;
    }

    public function setDateRange(DateRange $dateRange): TaskQuery
    {
        $this->dateRange = $dateRange;

        return $this;
    }

    public function getStatus(): array
    {
        return $this->status;
    }

    public function addStatus(string $status): TaskQuery
    {
        $this->status[] = $status;

        return $this;
    }

    public function setStatus(array $status): TaskQuery
    {
        $this->status = $status;

        return $this;
    }

    public function isUnassigned(): bool
    {
        return $this->unassigned;
    }

    public function setUnassigned(bool $unassigned): TaskQuery
    {
        $this->unassigned = $unassigned;

        return $this;
    }
}
