<?php

/*
 * This file is part of the TaskManagementBundle for Kimai 2.
 * All rights reserved by Kevin Papst (www.kevinpapst.de).
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace KimaiPlugin\TaskManagementBundle\Repository;

use App\Entity\Activity;
use App\Entity\Team;
use App\Entity\User;
use App\Repository\Paginator\LoaderPaginator;
use App\Repository\Paginator\PaginatorInterface;
use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\QueryBuilder;
use KimaiPlugin\TaskManagementBundle\Entity\Task;
use KimaiPlugin\TaskManagementBundle\Repository\Loader\TaskLoader;
use KimaiPlugin\TaskManagementBundle\Repository\Query\TaskQuery;
use Pagerfanta\Pagerfanta;

final class TaskRepository extends EntityRepository
{
    public function saveTask(Task $task)
    {
        $entityManager = $this->getEntityManager();
        $entityManager->persist($task);
        $entityManager->flush();
    }

    public function deleteTask(Task $task)
    {
        $entityManager = $this->getEntityManager();
        $entityManager->remove($task);
        $entityManager->flush();
    }

    public function getPagerfantaForQuery(TaskQuery $query): Pagerfanta
    {
        $paginator = new Pagerfanta($this->getPaginatorForQuery($query));
        $paginator->setMaxPerPage($query->getPageSize());
        $paginator->setCurrentPage($query->getPage());

        return $paginator;
    }

    protected function getPaginatorForQuery(TaskQuery $query): PaginatorInterface
    {
        $qb = $this->getQueryBuilderForQuery($query);
        $qb
            ->resetDQLPart('select')
            ->resetDQLPart('orderBy')
            ->select($qb->expr()->countDistinct('t.id'))
        ;
        $counter = (int) $qb->getQuery()->getSingleScalarResult();

        $qb = $this->getQueryBuilderForQuery($query);

        return new LoaderPaginator(new TaskLoader($qb->getEntityManager()), $qb, $counter);
    }

    /**
     * @param TaskQuery $query
     * @return Task[]
     */
    public function getTasksForQuery(TaskQuery $query): iterable
    {
        // this is using the paginator internally, as it will load all joined entities into the working unit
        // do not "optimize" to use the query directly, as it would results in hundreds of additional lazy queries
        $paginator = $this->getPaginatorForQuery($query);

        return $paginator->getAll();
    }

    private function getQueryBuilderForQuery(TaskQuery $query): QueryBuilder
    {
        $qb = $this->getEntityManager()->createQueryBuilder();

        $qb
            ->select('t')
            ->from(Task::class, 't')
            ->leftJoin('t.project', 'p')
            ->leftJoin('p.customer', 'c')
        ;

        if ($query->isUnassigned()) {
            $orUnassigned = $qb->expr()->orX(
                $qb->expr()->andX(
                    $qb->expr()->isNull('t.user'),
                    $qb->expr()->isNull('t.team')
                )
            );

            if (null !== $query->getCurrentUser()) {
                $orUnassigned->add(
                    $qb->expr()->andX(
                        $qb->expr()->isNull('t.user'),
                        $qb->expr()->in('t.team', ':userTeams')
                    )
                );
                $qb->setParameter('userTeams', $query->getCurrentUser()->getTeams());
            }
            $qb->andWhere($orUnassigned);
        } else {
            $user = [];

            if (null !== $query->getUser()) {
                $user[] = $query->getUser();
            }

            $user = array_merge($user, $query->getUsers());

            if (empty($user) && null !== $query->getCurrentUser()) {
                $currentUser = $query->getCurrentUser();

                if (!$currentUser->isSuperAdmin() && !$currentUser->isAdmin()) {
                    // make sure that the user is in the list of users, if he is part of a team
                    // if teams are used and the user is not a teamlead, the list of users would be empty and then leading to NOT limit the select by user IDs
                    $user[] = $currentUser;

                    /** @var Team $team */
                    foreach ($currentUser->getTeams() as $team) {
                        if ($currentUser->isTeamleadOf($team)) {
                            //$query->addTeam($team);
                            foreach ($team->getUsers() as $tmpUser) {
                                $user[] = $tmpUser;
                            }
                        }
                    }
                }
            }

            $user = array_map(function ($user) {
                if ($user instanceof User) {
                    return $user->getId();
                }

                return $user;
            }, $user);
            $user = array_unique($user);

            if (!empty($user)) {
                $qb->andWhere($qb->expr()->in('t.user', $user));
            }
        }

        /*
        if (null !== $query->getBegin()) {
            $orBegin = $qb->expr()->orX(
                $qb->expr()->lte('t.begin', ':begin'),
                $qb->expr()->isNull('t.begin')
            );
            $qb->andWhere($orBegin)->setParameter('begin', $query->getBegin());
        }
        */

        if (null !== $query->getEnd()) {
            $orEnd = $qb->expr()->orX(
                $qb->expr()->lte('t.end', ':end'),
                $qb->expr()->isNull('t.end')
            );
            $qb->andWhere($orEnd)->setParameter('end', $query->getEnd());
        }

        if (!empty($query->getStatus())) {
            $qb->andWhere(
                $qb->expr()->in('t.status', $query->getStatus())
            );
        }

        if (null !== $query->getActivity()) {
            $qb->andWhere('t.activity = :activity')
                ->setParameter('activity', $query->getActivity());
        }

        if (!empty($query->getTeams())) {
            $qb->andWhere($qb->expr()->in('t.team', ':team'))
                ->setParameter('team', $query->getTeams());
        }

        // only if no activity or a global activity was chosen
        if (null === $query->getActivity() || ($query->getActivity() instanceof Activity && null === $query->getActivity()->getProject())) {
            if (null !== $query->getProject()) {
                $qb->andWhere('t.project = :project')
                    ->setParameter('project', $query->getProject());
            } elseif (null !== $query->getCustomer()) {
                $qb->andWhere('p.customer = :customer')
                    ->setParameter('customer', $query->getCustomer());
            }
        }

        $orderBy = $query->getOrderBy();

        switch ($orderBy) {
            case 'project':
                $orderBy = 'p.name';
                break;
            case 'customer':
                $orderBy = 'c.name';
                break;
            case 'activity':
                $qb->leftJoin('t.activity', 'a');
                $orderBy = 'a.name';
                break;
            case 'user':
                $qb->leftJoin('t.user', 'u');
                $orderBy = 'u.username';
                break;
            case 'team':
                $qb->leftJoin('t.team', 'team');
                $orderBy = 'team.name';
                break;
            case 'end':
                $orderBy = 'endSort';
                if ($query->getOrder() === TaskQuery::ORDER_ASC) {
                    $qb->addSelect('COALESCE(t.end, \'9999-12-31 23:59:59\') AS HIDDEN endSort');
                } else {
                    $qb->addSelect('COALESCE(t.end, \'0000-12-31 23:59:59\') AS HIDDEN endSort');
                }
                break;
            default:
                $orderBy = 't.' . $orderBy;
                break;
        }

        $qb->addOrderBy($orderBy, $query->getOrder());

        if ($query->hasSearchTerm()) {
            $searchAnd = $qb->expr()->andX();
            $searchTerm = $query->getSearchTerm();

            if ($searchTerm->hasSearchTerm()) {
                $searchAnd->add(
                    $qb->expr()->orX(
                        $qb->expr()->like('t.title', ':searchTerm'),
                        $qb->expr()->like('t.description', ':searchTerm')
                    )
                );
                $qb->setParameter('searchTerm', '%' . $searchTerm->getSearchTerm() . '%');
            }

            if ($searchAnd->count() > 0) {
                $qb->andWhere($searchAnd);
            }
        }

        return $qb;
    }
}
