<?php

/*
 * This file is part of the TaskManagementBundle for Kimai 2.
 * All rights reserved by Kevin Papst (www.kevinpapst.de).
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace KimaiPlugin\TaskManagementBundle\Repository;

use App\Entity\Timesheet;
use App\Entity\User;
use App\Repository\Query\TimesheetQuery;
use App\Timesheet\TimesheetService;
use KimaiPlugin\TaskManagementBundle\Entity\Task;

final class TaskService
{
    /**
     * @var TaskRepository
     */
    private $tasks;
    /**
     * @var TimesheetService
     */
    private $timesheets;

    public function __construct(TaskRepository $task, TimesheetService $timesheet)
    {
        $this->tasks = $task;
        $this->timesheets = $timesheet;
    }

    public function findTask($id): ?Task
    {
        return $this->tasks->find($id);
    }

    public function startTask(Task $task)
    {
        $timesheet = new Timesheet();
        $timesheet
            ->setUser($task->getUser())
            ->setProject($task->getProject())
            ->setActivity($task->getActivity())
            ->setDescription($task->getDescription())
        ;

        $this->timesheets->prepareNewTimesheet($timesheet);
        $this->timesheets->saveNewTimesheet($timesheet);

        $task->addTimesheet($timesheet);
        $task->setStatus(Task::STATUS_IN_PROGRESS);
        $this->tasks->saveTask($task);
    }

    public function stopTask(Task $task)
    {
        $timesheet = $task->getActiveRecord();

        if (null === $timesheet) {
            throw new \InvalidArgumentException('Given task is not running');
        }

        $this->timesheets->stopTimesheet($timesheet);
    }

    private function stopActiveRecord(Task $task)
    {
        $timesheet = $task->getActiveRecord();

        if (null !== $timesheet) {
            $this->timesheets->stopTimesheet($timesheet);
        }
    }

    public function deleteTask(Task $task)
    {
        $this->stopActiveRecord($task);
        $this->tasks->deleteTask($task);
    }

    public function closeTask(Task $task)
    {
        $this->stopActiveRecord($task);

        $task->setStatus(Task::STATUS_DONE);
        $this->tasks->saveTask($task);
    }

    public function reopenTask(Task $task)
    {
        $task->setStatus(Task::STATUS_IN_PROGRESS);
        $this->tasks->saveTask($task);
    }

    public function unassignTask(Task $task)
    {
        $this->stopActiveRecord($task);

        $task->setUser(null);
        $task->setStatus(Task::STATUS_PENDING);
        $this->tasks->saveTask($task);
    }

    public function assignTask(Task $task, User $user)
    {
        $task->setUser($user);
        $task->setStatus(Task::STATUS_IN_PROGRESS);
        $this->tasks->saveTask($task);
    }

    private function findTimesheetsForTask(Task $task)
    {
        $query = new TimesheetQuery();
        $query->setProject($task->getProject());
        $query->setActivity($task->getActivity());
        $query->setState(TimesheetQuery::STATE_RUNNING);
    }
}
